#include <Wire.h>
#include <Adafruit_VL53L0X.h>

// Define the vibration pins
#define vibrationPin1 11
#define vibrationPin2 12
#define vibrationPin3 13  // New vibration motor pin for the third sensor

// Fixed duration for vibration (in milliseconds)
#define vibrationDuration 100  // 100 ms
unsigned long previousVibrationTime1 = 0;
unsigned long previousVibrationTime2 = 0;
unsigned long previousVibrationTime3 = 0; // New variable for the third vibration motor

// Define sensor addresses
#define LOX1_ADDRESS 0x30
#define LOX2_ADDRESS 0x31
#define LOX3_ADDRESS 0x32 // Address for the third sensor

// Define shutdown pins
#define SHT_LOX1 7
#define SHT_LOX2 6
#define SHT_LOX3 5 // Shutdown pin for third sensor

// Create sensor objects
Adafruit_VL53L0X lox1 = Adafruit_VL53L0X();
Adafruit_VL53L0X lox2 = Adafruit_VL53L0X();
Adafruit_VL53L0X lox3 = Adafruit_VL53L0X(); // Third sensor

// Measurement data structures (minimize memory usage)
uint16_t measure1Distance = 0;
uint16_t measure2Distance = 0;
uint16_t measure3Distance = 0; // New measurement data for third sensor

// Function to initialize the sensors
void setID() {
  // Reset all sensors
  digitalWrite(SHT_LOX1, LOW);
  digitalWrite(SHT_LOX2, LOW);
  digitalWrite(SHT_LOX3, LOW);
  delay(10);
  
  // Bring all sensors out of reset
  digitalWrite(SHT_LOX1, HIGH);
  digitalWrite(SHT_LOX2, HIGH);
  digitalWrite(SHT_LOX3, HIGH);
  delay(10);

  // Initialize sensor 1
  if (!lox1.begin(LOX1_ADDRESS)) {
   
    while (1);
  }
  delay(10);

  // Initialize sensor 2
  if (!lox2.begin(LOX2_ADDRESS)) {
   
    while (1);
  }
  delay(10);

  // Initialize sensor 3
  if (!lox3.begin(LOX3_ADDRESS)) {
    
    while (1);
  }
}

// Function to read all three sensors
void read_dual_sensors() {
  VL53L0X_RangingMeasurementData_t measureData;
  
  // Read distances from the sensors
  lox1.rangingTest(&measureData, false);
  measure1Distance = (measureData.RangeStatus != 4) ? measureData.RangeMilliMeter : 0;

  lox2.rangingTest(&measureData, false);
  measure2Distance = (measureData.RangeStatus != 4) ? measureData.RangeMilliMeter : 0;

  lox3.rangingTest(&measureData, false);
  measure3Distance = (measureData.RangeStatus != 4) ? measureData.RangeMilliMeter : 0;
  
}

// Function to control the vibration motors
void vibration_start() {
  unsigned long vibrationInterval1 = 0;
  unsigned long vibrationInterval2 = 0;
  unsigned long vibrationInterval3 = 0; // New variable for the third sensor's vibration motor

  // Sensor 1
  if (measure1Distance > 2438) vibrationInterval1 = 0;
  else if (measure1Distance > 2133) vibrationInterval1 = 3500;
  else if (measure1Distance > 1829) vibrationInterval1 = 3000;
  else if (measure1Distance > 1524) vibrationInterval1 = 2500;
  else if (measure1Distance > 1219) vibrationInterval1 = 2000;
  else if (measure1Distance > 914) vibrationInterval1 = 1500;
  else if (measure1Distance > 610) vibrationInterval1 = 1000;
  else if (measure1Distance > 457) vibrationInterval1 = 750;
  else if (measure1Distance > 305) vibrationInterval1 = 500;
  else if (measure1Distance > 152) vibrationInterval1 = 250;
  else if (measure1Distance > 0) vibrationInterval1 = 100;

  // Sensor 2
  if (measure2Distance > 2438) vibrationInterval2 = 0;
  else if (measure2Distance > 2133) vibrationInterval2 = 3500;
  else if (measure2Distance > 1829) vibrationInterval2 = 3000;
  else if (measure2Distance > 1524) vibrationInterval2 = 2500;
  else if (measure2Distance > 1219) vibrationInterval2 = 2000;
  else if (measure2Distance > 914) vibrationInterval2 = 1500;
  else if (measure2Distance > 610) vibrationInterval2 = 1000;
  else if (measure2Distance > 457) vibrationInterval2 = 750;
  else if (measure2Distance > 305) vibrationInterval2 = 500;
  else if (measure2Distance > 152) vibrationInterval2 = 250;
  else if (measure2Distance > 0) vibrationInterval2 = 100;

  // Sensor 3
  if (measure3Distance > 2438) vibrationInterval3 = 0;
  else if (measure3Distance > 2133) vibrationInterval3 = 3500;
  else if (measure3Distance > 1829) vibrationInterval3 = 3000;
  else if (measure3Distance > 1524) vibrationInterval3 = 2500;
  else if (measure3Distance > 1219) vibrationInterval3 = 2000;
  else if (measure3Distance > 914) vibrationInterval3 = 1500;
  else if (measure3Distance > 610) vibrationInterval3 = 1000;
  else if (measure3Distance > 457) vibrationInterval3 = 750;
  else if (measure3Distance > 305) vibrationInterval3 = 500;
  else if (measure3Distance > 152) vibrationInterval3 = 250;
  else if (measure3Distance > 0) vibrationInterval3 = 100;

  // Control vibration for Sensor 1
  if (millis() - previousVibrationTime1 >= vibrationInterval1) {
    digitalWrite(vibrationPin1, HIGH); // Turn on vibration motor
    previousVibrationTime1 = millis(); // Update the last vibration time
  } else if (millis() - previousVibrationTime1 >= vibrationDuration) {
    digitalWrite(vibrationPin1, LOW); // Turn off vibration motor
  }

  // Control vibration for Sensor 2
  if (millis() - previousVibrationTime2 >= vibrationInterval2) {
    digitalWrite(vibrationPin2, HIGH); // Turn on vibration motor
    previousVibrationTime2 = millis(); // Update the last vibration time
  } else if (millis() - previousVibrationTime2 >= vibrationDuration) {
    digitalWrite(vibrationPin2, LOW); // Turn off vibration motor
  }

  // Control vibration for Sensor 3
  if (millis() - previousVibrationTime3 >= vibrationInterval3) {
    digitalWrite(vibrationPin3, HIGH); // Turn on vibration motor
    previousVibrationTime3 = millis(); // Update the last vibration time
  } else if (millis() - previousVibrationTime3 >= vibrationDuration) {
    digitalWrite(vibrationPin3, LOW); // Turn off vibration motor
  }
}

void setup() {
  Serial.begin(115200);
  pinMode(vibrationPin1, OUTPUT);
  pinMode(vibrationPin2, OUTPUT);
  pinMode(vibrationPin3, OUTPUT);
  
  pinMode(SHT_LOX1, OUTPUT);
  pinMode(SHT_LOX2, OUTPUT);
  pinMode(SHT_LOX3, OUTPUT);
  
  
  setID(); // Initialize the sensors
}

void loop() {
  read_dual_sensors(); // Read distances from all sensors
  vibration_start(); // Control vibration motors based on distance
}
